function [H, sigma2, logl] = dcv_qn(X, v, H_0, sigma2_0, FMIN_OPT, VAR_OFFSET);

%dcv_qn    Blind identification using quasi Newton optimization.
%Use:	[H,sigma2,logl] = dcv_qn(X,v,H_0,sigma2_0[,FMIN_OPT,VAR_OFFSET]).
%	Returns the estimated quantities corresponding to the last iteration.
%	FMIN_OPT may be used to pass options to fminunc (Optimization Toolbox
%	is required for using this function). VAR_OFFSET is a threshold that
%	prevents numerical underflows (can be set to zero is the SNR is high
%	and/or if the algorithm is started from a point close to a local
%	optimum of the likelihood function).

% DCV Toolbox, $Revision: 1.2 $
% Olivier Capp, 10/03/98 - $Date: 2004/11/19 10:57:16 $
% http://www.tsi.enst.fr/~cappe/dcv/

% Check that Optimization Toolbox is present
if (exist('fminunc') ~= 2)
  error('Optimization toolbox (fminu) is required.');
end;

% Input args.
error(nargchk(4, 6, nargin));
% Set parameters for the DCV functions
[AR,AC,V]=dcv_init(v,length(H_0));
[NStat,NSymb,LFilt,REAL_COEFF]=dcv_chk(AR,AC,V,H_0);
if (nargin < 5)
  FMIN_OPT = optimset;	% Use default options
end;

% Set a variance offset that corresponds to a maximum SNR of 25 dB to avoid
% numerical problems (obviously needs to be changed is the SNR is larger than
% that!)
if (nargin < 6)
  VAR_OFFSET = cov(X)*(10^(-2.5));
end;

if (REAL_COEFF)
  % Use fminunc for optimization
  % Input arguments
  args = [H_0; log(max(sigma2_0-VAR_OFFSET,eps))];
  args = fminunc('dcv_qn_f', args, FMIN_OPT, X, AR, AC, V, ...
    VAR_OFFSET, REAL_COEFF);
  % Ouput arguments
  H = args(1:LFilt);
  sigma2 = exp(args(LFilt+1))+VAR_OFFSET;
else
  % Use fminunc for optimization taking care of the fact that fminunc does not
  % support complex arguments
  % Input arguments
  args = [real(H_0); imag(H_0); log(max(sigma2_0-VAR_OFFSET,eps))];
  args = fminunc('dcv_qn_f', args, FMIN_OPT, X, AR, AC, V, ...
    VAR_OFFSET, REAL_COEFF);
  % Ouput arguments
  H = args(1:LFilt) + i*args(1+LFilt:2*LFilt);
  sigma2 = exp(args(2*LFilt+1))+VAR_OFFSET;
end;

% Echo
fprintf(1,'\n');

% Compute loglikelhood if needed
if (nargout > 2)
  logl = dcv_llk (X, AR, AC, V, H, sigma2);
end;
